/**********************************************************************
 Copyright (c) 1996, 1997 Christopher Unkel.  All rights reserved.
 
 Use and copying of the software and preparation and distribution of
 derivative works is permitted provided this copyright notice is
 left intact.  Any distribution of this software or derivative works
 must comply with all applicable United States export control laws.
 
 This software is provided AS IS, without express or implied warranty
 of any kind.  Christopher Unkel makes no representations concerning
 either the merchantability of this software or the suitability of
 this software of any particular purpose.
 **********************************************************************/

/* Random number generation, taken mostly directly from Robert Sedgewick's
   _Algorithms in C_. */

#include "ac-random.h"
#include <limits.h>
#include <windows.h>

#define M  2147395600U
#define M1 46340U

#define B  34078421

static unsigned a[55];
static unsigned j;

static unsigned mult (unsigned a, unsigned b);

unsigned
ac_random( unsigned r )
{
  j = (j+1) % 55;
  a[j] = (a[(j+23) % 55] + a[(j+54) % 55]) % M;
  return MulDiv(a[j], r, M) % r;
}

void
ac_srandom( unsigned s )
{
  s = s % M;
  for (a[0]=s, j=1; j < 55; j++)
    a[j] = (mult(a[j-1], B)+1) % M;
}

#define MU  2147395600U
#define M1U 46340U

static unsigned
mult (unsigned p, unsigned q)
{
  unsigned p1, p0, q1, q0;
  p1=p/M1; p0=p%M1;
  q1=q/M1; q0=q%M1;
  return ((((p0*q1+p1*q0) % M1U)*M1U+p0*q0) % MU);
}


